import click
import importlib_metadata
from pathlib import Path
from .core import generate_context, run_doctor

def get_version():
    try:
        return importlib_metadata.version('dircontextgen')
    except importlib_metadata.PackageNotFoundError:
        return '0.0.0'

@click.command()
@click.option(
    '--path', '-p',
    default='.',
    type=click.Path(exists=True, file_okay=False, dir_okay=True, readable=True),
    help='Directory path to process (default: current directory)'
)
@click.option(
    '--output', '-o',
    type=click.Path(dir_okay=False, writable=True),
    help='Output file path (default: stdout)'
)
@click.option(
    '--doctor',
    is_flag=True,
    help='Run diagnostics on ignore/target/peek patterns'
)
@click.option(
    '--debug', '-d',
    is_flag=True,
    help='Enable debug logging'
)
@click.option(
    '--gui', '-g',
    is_flag=True,
    help='Launch GUI interface for modifying files based on ignore/target/peek patterns (WIP)'
)
@click.version_option(
    get_version(),
    '--version', '-v',
    message='dircontextgen version %(version)s'
)
@click.option(
    '--max-depth', '-m',
    type=int,
    help='Maximum directory depth to process'
)
def cli(path, output, doctor, debug, gui, max_depth):
    """
    Generate structured markdown documentation of a directory, combining
    .gitignore and .dirfilter rules for precise control
    over included content.
    """
    if gui:
        try:
            from .gui import launch_gui
            launch_gui(
                start_path=path,
                debug=debug
            )
            return
        except ImportError as e:
            click.secho("Error: PySide6 is required for GUI mode. Install with 'pip install PySide6'", 
                       fg='red', err=True)
            raise click.Abort()
        except Exception as e:
            if debug:
                raise
            click.secho(f"Error launching GUI: {str(e)}", fg='red', err=True)
            raise click.Abort()

    if doctor:
        run_doctor(start_path=path, debug=debug)
        click.secho("✓ Pattern diagnostics completed", fg='green')
    else:
        try:
            # Get relative output path if specified
            output_path = None
            if output:
                start = Path(path).resolve()
                output_path = str(Path(output).resolve().relative_to(start))
            
            result = generate_context(
                start_path=path,
                output_file=output_path,
                max_depth=max_depth,
                debug=debug
            )
                    
            if output:
                with open(output, 'w', encoding='utf-8') as f:
                    f.write(result)
                click.secho(f"✓ Documentation saved to {output}", fg='green')
            else:
                click.echo(result)
        except Exception as e:
            if debug:
                raise
            click.secho(f"Error: {str(e)}", fg='red', err=True)
            raise click.Abort()

if __name__ == '__main__':
    cli()