import os
import io
import mimetypes
from pathlib import Path
from typing import List, Dict, Tuple, Optional, Set
from pathspec import PathSpec
from pathspec.patterns.gitwildmatch import GitWildMatchPattern

class DirectoryProcessor:
    def __init__(self, root_dir: Path, output_file: Optional[Path], max_depth: Optional[int], debug: bool):
        self.root_dir = root_dir.resolve()
        self.output_file = output_file.resolve() if output_file else None
        self.max_depth = max_depth
        self.debug = debug
        self.exclude_specs: List[Tuple[Path, PathSpec]] = []
        self.include_specs: List[Tuple[Path, PathSpec]] = []
        self.peek_specs: List[Tuple[Path, PathSpec]] = []
        self.debug_log: List[str] = []
        self.dirfilter_files: List[Path] = []

        # Add default .git exclusion
        self._add_default_excludes()
        self._load_pattern_files()
        self._add_output_excludes()

        if self.debug:
            self._log_pre_filter_tree()

    def _add_default_excludes(self):
        """Add default exclusions like .git directory"""
        # Change from '.git/' to '**/.git/' to match at any depth
        self.exclude_specs.append(
            (self.root_dir, PathSpec.from_lines(GitWildMatchPattern, ['**/.git/']))
        )
        if self.debug:
            self.debug_log.append("Added default exclusion: **/.git/")

    def _log_pre_filter_tree(self):
        """Generate directory tree before any filtering"""
        self.debug_log.append("\nFull directory tree before filtering:")
        
        def walk_dir(dir_path: Path, prefix: str = ''):
            try:
                entries = sorted(dir_path.iterdir(), key=lambda e: (not e.is_dir(), e.name.lower()))
            except PermissionError:
                return

            for i, entry in enumerate(entries):
                is_last = i == len(entries) - 1
                connector = '└── ' if is_last else '├── '
                new_prefix = prefix + ('    ' if is_last else '│   ')
                
                self.debug_log.append(f"{prefix}{connector}{entry.name}")
                
                if entry.is_dir():
                    walk_dir(entry, new_prefix)
        
        walk_dir(self.root_dir)
        
    def _load_pattern_files(self):
        """Load patterns from .gitignore and .dirfilter files"""
        for dirpath, _, files in os.walk(self.root_dir):
            current_dir = Path(dirpath)
            
            # Process .gitignore files
            if '.gitignore' in files:
                gitignore = current_dir / '.gitignore'
                with gitignore.open('r', encoding='utf-8') as f:
                    patterns = []
                    for line in f:
                        line = line.strip()
                        if line and not line.startswith('#'):
                            # Don't automatically add **/ prefix to .gitignore patterns
                            # Only handle leading slash removal
                            if line.startswith('/'):
                                patterns.append(line[1:])
                            else:
                                patterns.append(line)
                
                if patterns:
                    if self.debug:
                        self.debug_log.append(f"Found .gitignore at {gitignore.relative_to(self.root_dir)}")
                        self.debug_log.extend([f"  - {p}" for p in patterns])
                    spec = PathSpec.from_lines(GitWildMatchPattern, patterns)
                    self.exclude_specs.append((current_dir, spec))
            
            # Process .dirfilter files
            if '.dirfilter' in files:
                self._process_dirfilter(current_dir)
            
    def _log_pattern_files(self):
        """Log all found pattern files"""
        self.debug_log.append("\nFound pattern files:")
        for path, spec in self.exclude_specs:
            if path.name == '.gitignore':
                self.debug_log.append(f" - .gitignore at {path.relative_to(self.root_dir)}")
        for df in self.dirfilter_files:
            self.debug_log.append(f" - .dirfilter at {df.relative_to(self.root_dir)}")

    def _normalize_path(self, path_str: str) -> str:
        """Normalize path string to use forward slashes"""
        return path_str.replace('\\', '/')

    def _process_dirfilter(self, directory: Path) -> None:
        """Process .dirfilter file in the given directory
        
        Args:
            directory (Path): Directory containing the .dirfilter file
        """
        dirfilter = directory / '.dirfilter'
        if not dirfilter.exists() or not dirfilter.is_file():
            return

        self.dirfilter_files.append(dirfilter)
        sections = {
            'ignore': [],
            'target': [],
            'peek': []
        }
        current_section = None

        try:
            with dirfilter.open('r', encoding='utf-8') as f:
                for line_num, line in enumerate(f, 1):
                    line = line.split('#', 1)[0].strip()
                    if not line:
                        continue
                        
                    if line.startswith('[') and line.endswith(']'):
                        section = line[1:-1].lower()
                        if section not in sections:
                            if self.debug:
                                self.debug_log.append(
                                    f"Warning: Unknown section '{section}' in "
                                    f"{dirfilter.relative_to(self.root_dir)} line {line_num}"
                                )
                            continue
                        current_section = section
                        continue
                        
                    if current_section in sections:
                        # For target patterns, ensure they're absolute relative to root
                        if current_section == 'target':
                            # If pattern doesn't start with /, make it relative to root
                            if not line.startswith('/'):
                                line = '/' + line
                            # Convert Windows paths to forward slashes
                            line = line.replace('\\', '/')
                            # Remove leading slash for pattern matching
                            if line.startswith('/'):
                                line = line[1:]
                        else:
                            # For other patterns, just normalize slashes
                            line = line.replace('\\', '/')
                        
                        sections[current_section].append(line)
                    elif self.debug:
                        self.debug_log.append(
                            f"Warning: Pattern '{line}' outside of any section in "
                            f"{dirfilter.relative_to(self.root_dir)} line {line_num}"
                        )

            if self.debug:
                rel_path = dirfilter.relative_to(self.root_dir)
                self.debug_log.append(f"\nPatterns from {rel_path}:")
                for section, patterns in sections.items():
                    if patterns:
                        self.debug_log.append(f"  [{section}]")
                        for pattern in patterns:
                            self.debug_log.append(f"    {pattern}")

            # Process each section's patterns
            for section, patterns in sections.items():
                if not patterns:
                    continue
                    
                try:
                    # For target patterns, create a more specific PathSpec
                    if section == 'target':
                        # Make patterns more specific to match exact files
                        exact_patterns = []
                        for pattern in patterns:
                            if not pattern.endswith('/'):
                                exact_patterns.append(pattern)
                        if exact_patterns:
                            spec = PathSpec.from_lines(GitWildMatchPattern, exact_patterns)
                            self.include_specs.append((self.root_dir, spec))
                    else:
                        spec = PathSpec.from_lines(GitWildMatchPattern, patterns)
                        if section == 'ignore':
                            self.exclude_specs.append((directory, spec))
                        elif section == 'peek':
                            self.peek_specs.append((directory, spec))
                except Exception as e:
                    if self.debug:
                        self.debug_log.append(
                            f"Error creating PathSpec for {section} patterns in "
                            f"{dirfilter.relative_to(self.root_dir)}: {str(e)}"
                        )

        except Exception as e:
            if self.debug:
                self.debug_log.append(
                    f"Error processing {dirfilter.relative_to(self.root_dir)}: {str(e)}"
                )
            
    def _add_output_excludes(self):
        """Exclude the output file from processing"""
        if self.output_file and self.output_file.is_relative_to(self.root_dir):
            output_rel = self.output_file.relative_to(self.root_dir)
            patterns = [str(output_rel)]
            
            # Exclude all parent directories of the output file
            for parent in output_rel.parents:
                patterns.append(f"{str(parent)}/*")
                
            spec = PathSpec.from_lines(GitWildMatchPattern, patterns)
            self.exclude_specs.append((self.root_dir, spec))
            if self.debug:
                self.debug_log.append(f"\nExcluding output file: {output_rel}")
                self.debug_log.extend([f"  - {p}" for p in patterns])

    def _is_excluded(self, path: Path) -> bool:
        """Check if a path should be excluded.
        
        Args:
            path (Path): Path to check for exclusion
            
        Returns:
            bool: True if the path should be excluded, False otherwise
        """
        if not path.is_relative_to(self.root_dir):
            return False

        # Normalize path for pattern matching
        path_str = str(path.relative_to(self.root_dir)).replace('\\', '/')
        is_dir = path.is_dir()

        if self.debug:
            self.debug_log.append(f"\nExclusion check for: {path_str}")

        # First check if the path is explicitly targeted for inclusion
        # This takes highest precedence and can override any exclusions
        for inc_base, inc_spec in self.include_specs:
            if path.is_relative_to(inc_base):
                inc_rel = str(path.relative_to(inc_base)).replace('\\', '/')
                if is_dir and not inc_rel.endswith('/'):
                    inc_rel += '/'
                if inc_spec.match_file(inc_rel):
                    if self.debug:
                        self.debug_log.append(f"  → Explicitly targeted for inclusion: {inc_rel}")
                    return False

        # Track if path is excluded by directory patterns
        excluded_by_dir = False
        excluding_dir_pattern = None
        
        # Check directory-based exclusions first
        for base_dir, spec in self.exclude_specs:
            if not path.is_relative_to(base_dir):
                continue
                
            rel = str(path.relative_to(base_dir)).replace('\\', '/')
            if is_dir and not rel.endswith('/'):
                rel += '/'

            # Check parent directories for exclusion patterns
            parts = Path(rel).parts
            for i in range(len(parts)):
                parent = '/'.join(parts[:i+1])
                if spec.match_file(parent + '/'):  # Add trailing slash to match directory patterns
                    excluded_by_dir = True
                    excluding_dir_pattern = parent
                    if self.debug:
                        self.debug_log.append(f"  → Parent directory excluded by pattern: {parent}/")
                    break

            # Check direct file exclusion
            if spec.match_file(rel):
                if self.debug:
                    matching_patterns = [p.pattern for p in spec.patterns if p.match_file(rel)]
                    self.debug_log.append(f"  → Directly excluded by patterns: {matching_patterns}")
                return True

        # If excluded by directory, check for target override
        if excluded_by_dir:
            for inc_base, inc_spec in self.include_specs:
                if path.is_relative_to(inc_base):
                    inc_rel = str(path.relative_to(inc_base)).replace('\\', '/')
                    if is_dir and not inc_rel.endswith('/'):
                        inc_rel += '/'
                    
                    # Allow target pattern to override directory exclusion
                    if inc_spec.match_file(inc_rel):
                        if self.debug:
                            self.debug_log.append(
                                f"  → Directory exclusion ({excluding_dir_pattern}/) "
                                f"overridden by target pattern for: {inc_rel}"
                            )
                        return False
            return True

        # Check peek patterns last
        for base_dir, spec in self.peek_specs:
            if not path.is_relative_to(base_dir):
                continue
                
            rel = str(path.relative_to(base_dir)).replace('\\', '/')
            
            if spec.match_file(rel):
                if self.debug:
                    self.debug_log.append(f"  → Peek match: {rel}")
                return False
                
            # Check parent directories
            parts = Path(rel).parts
            for i in range(len(parts)):
                parent = '/'.join(parts[:i+1])
                if spec.match_file(parent):
                    if self.debug:
                        self.debug_log.append(f"  → Inside peeked directory: {parent}")
                    return False

        return False

    def _is_peek(self, path: Path) -> bool:
        """Check if a path should have hidden content
        
        Args:
            path (Path): File path to check
            
        Returns:
            bool: True if the file's contents should be hidden
        """
        # Only files can be peeked
        if not path.is_file():
            return False

        # Don't peek excluded files
        if self._is_excluded(path):
            return False

        try:
            for base_dir, spec in self.peek_specs:
                # Only check patterns if the path is under the base directory
                if path.is_relative_to(base_dir):
                    # Get path relative to the base directory where the peek pattern was defined
                    rel_path = str(path.relative_to(base_dir)).replace('\\', '/')
                    
                    # Direct match check
                    if spec.match_file(rel_path):
                        if self.debug:
                            matching_patterns = [
                                pattern.pattern 
                                for pattern in spec.patterns 
                                if pattern.match_file(rel_path)
                            ]
                            self.debug_log.append(
                                f"Peek match: {path.relative_to(self.root_dir)} matched "
                                f"patterns {matching_patterns} in {base_dir.relative_to(self.root_dir)}"
                            )
                        return True
                    
                    # Check if any parent directory matches peek patterns
                    # This handles cases where a parent directory is peeked
                    parts = Path(rel_path).parts
                    for i in range(len(parts)):
                        parent = '/'.join(parts[:i+1])
                        if spec.match_file(parent):
                            if self.debug:
                                self.debug_log.append(
                                    f"File {path.relative_to(self.root_dir)} is inside "
                                    f"peeked directory: {parent}"
                                )
                            return True
        except Exception as e:
            if self.debug:
                self.debug_log.append(f"Error in _is_peek for {path}: {str(e)}")
            return False

        return False
    
    @staticmethod
    def _is_binary(file_path: Path) -> bool:
        """Check if a file is binary"""
        try:
            with file_path.open('rb') as f:
                chunk = f.read(1024)
                return b'\0' in chunk
        except IOError:
            return True

    def _build_tree(self) -> Tuple[List[str], List[str]]:
        """Build directory tree and collect file contents
        
        Returns:
            Tuple containing:
            - List[str]: Lines representing the directory tree structure
            - List[str]: File contents with headers and separators
        """
        tree = []
        file_contents = []
        separator = '-' * 80

        def check_for_target_files(dir_path: Path) -> bool:
            """Check if directory contains any targeted files"""
            try:
                for entry in dir_path.iterdir():
                    if entry.is_file():
                        rel_path = str(entry.relative_to(self.root_dir)).replace('\\', '/')
                        for base_dir, spec in self.include_specs:
                            if entry.is_relative_to(base_dir):
                                inc_rel = str(entry.relative_to(base_dir)).replace('\\', '/')
                                if spec.match_file(inc_rel):
                                    return True
                    elif entry.is_dir():
                        if check_for_target_files(entry):
                            return True
                return False
            except PermissionError:
                return False

        def should_include_directory(dir_path: Path) -> bool:
            """Check if directory should be included in output"""
            # Always include directory if it contains target files
            if check_for_target_files(dir_path):
                if self.debug:
                    self.debug_log.append(f"Including directory {dir_path} due to targeted files")
                return True
                
            # Check if directory itself is excluded
            if self._is_excluded(dir_path):
                if self.debug:
                    self.debug_log.append(f"Excluding directory {dir_path}")
                return False
                
            # Include if it has any non-excluded content
            try:
                return any(
                    not self._is_excluded(child) or 
                    (child.is_dir() and should_include_directory(child))
                    for child in dir_path.iterdir()
                )
            except PermissionError:
                return False

        def is_targeted_file(file_path: Path) -> bool:
            """Check if file is explicitly targeted"""
            rel_path = str(file_path.relative_to(self.root_dir)).replace('\\', '/')
            for base_dir, spec in self.include_specs:
                if file_path.is_relative_to(base_dir):
                    inc_rel = str(file_path.relative_to(base_dir)).replace('\\', '/')
                    if spec.match_file(inc_rel):
                        if self.debug:
                            self.debug_log.append(f"File {rel_path} is targeted")
                        return True
            return False

        def process_directory(current_dir: Path, prefix: str = '', depth: int = 0):
            """Process a directory and its contents"""
            if self.max_depth and depth > self.max_depth:
                return

            try:
                if self.debug:
                    self.debug_log.append(f"\nProcessing directory: {current_dir.relative_to(self.root_dir)}")

                # Get all entries and sort them
                try:
                    all_entries = sorted(current_dir.iterdir(), key=lambda e: (not e.is_dir(), e.name.lower()))
                    entries = []
                    
                    # First pass: collect entries to include
                    for entry in all_entries:
                        include = False
                        
                        if entry.is_dir():
                            # Include directory if it contains target files or has non-excluded content
                            if should_include_directory(entry):
                                include = True
                        else:
                            # Include file if it's targeted or not excluded
                            if is_targeted_file(entry) or not self._is_excluded(entry):
                                include = True
                                
                        if include:
                            entries.append(entry)

                    # Second pass: process included entries
                    for i, entry in enumerate(entries):
                        is_last = i == len(entries) - 1
                        connector = '└── ' if is_last else '├── '
                        new_prefix = prefix + ('    ' if is_last else '│   ')
                        
                        # Add entry to tree
                        tree.append(f"{prefix}{connector}{entry.name}")

                        if entry.is_dir():
                            process_directory(entry, new_prefix, depth + 1)
                        elif entry.is_file():
                            # Process file content
                            rel_path = str(entry.relative_to(self.root_dir))
                            if self._is_peek(entry):
                                file_contents.append(
                                    f"\nFile: {rel_path}\n\n"
                                    f"{separator}\n\n"
                                    f"[Contents hidden]\n\n"
                                    f"{separator}\n"
                                )
                            elif self._is_binary(entry):
                                file_contents.append(
                                    f"\nFile: {rel_path}\n\n"
                                    f"{separator}\n\n"
                                    f"[Binary file]\n\n"
                                    f"{separator}\n"
                                )
                            else:
                                try:
                                    with entry.open('r', encoding='utf-8') as f:
                                        content = f.read().rstrip()
                                    file_contents.append(
                                        f"\nFile: {rel_path}\n\n"
                                        f"{separator}\n\n"
                                        f"{content}\n\n"
                                        f"{separator}\n"
                                    )
                                except Exception as e:
                                    if self.debug:
                                        self.debug_log.append(f"Error reading {rel_path}: {str(e)}")
                                    file_contents.append(
                                        f"\nFile: {rel_path}\n\n"
                                        f"{separator}\n\n"
                                        f"[Error reading file]\n\n"
                                        f"{separator}\n"
                                    )

                except PermissionError:
                    if self.debug:
                        self.debug_log.append(f"Permission denied: {current_dir}")
                    return
                    
            except Exception as e:
                if self.debug:
                    self.debug_log.append(f"Error processing directory {current_dir}: {str(e)}")
                return

        process_directory(self.root_dir)
        return tree, file_contents

    def generate_documentation(self) -> str:
        """Main documentation generation method"""
        output = ["# Project Documentation\n"]

        # Debug information
        if self.debug:
            output.append("## Debug Information\n```")
            output.extend(self.debug_log)
            output.append("```\n")

        # Directory structure
        output.append("\n## Directory Structure\n```\n")
        tree, file_contents = self._build_tree()
        output.extend(tree)
        output.append("\n```\n")

        # File contents
        output.append("\n## File Contents\n")
        output.extend(file_contents)

        return '\n'.join(output)


def generate_context(
    start_path: str,
    output_file: Optional[str] = None,
    max_depth: Optional[int] = None,
    debug: bool = False
) -> str:
    root_dir = Path(start_path).resolve()
    output_path = Path(output_file).resolve() if output_file else None

    processor = DirectoryProcessor(
        root_dir=root_dir,
        output_file=output_path,
        max_depth=max_depth,
        debug=debug
    )

    return processor.generate_documentation()


def run_doctor(start_path: str, debug: bool = False):
    """Diagnostics implementation (placeholder)"""
    pass