# project_tree.py
import os
from pathlib import Path
from typing import List, Dict, Optional, Tuple
from dataclasses import dataclass, field
from pathspec import PathSpec
from pathspec.patterns.gitwildmatch import GitWildMatchPattern

@dataclass
class ProjectTreeNode:
    """Represents a file or directory in the project tree"""
    path: Path
    is_dir: bool
    is_excluded: bool = False
    is_targeted: bool = False
    is_peeked: bool = False
    children: List['ProjectTreeNode'] = field(default_factory=list)
    content: Optional[str] = None

class ProjectTreeBuilder:
    def __init__(self, root_dir: Path, max_depth: Optional[int] = None, debug: bool = False):
        """
        Initialize the project tree builder
        
        Args:
            root_dir (Path): Root directory to process
            max_depth (Optional[int]): Maximum depth to process directories
            debug (bool): Enable debug logging
        """
        self.root_dir = root_dir.resolve()
        self.max_depth = max_depth
        self.debug = debug
        
        # Pattern collections
        self.exclude_specs: List[Tuple[Path, PathSpec]] = []
        self.include_specs: List[Tuple[Path, PathSpec]] = []
        self.peek_specs: List[Tuple[Path, PathSpec]] = []
        
        # Debug logging
        self.debug_log: List[str] = []
        
        # Default exclusions
        self._add_default_excludes()
        
        # Load pattern files
        self._load_pattern_files()

    def _add_default_excludes(self):
        """Add default exclusions like .git directory"""
        self.exclude_specs.append(
            (self.root_dir, PathSpec.from_lines(GitWildMatchPattern, ['**/.git/']))
        )
        if self.debug:
            self.debug_log.append("Added default exclusion: **/.git/")

    def _load_pattern_files(self):
        """Load patterns from .gitignore and .dirfilter files"""
        # Similar to the existing implementation in core.py
        # Will include pattern loading logic from the current core.py _load_pattern_files method
        pass

    def _is_excluded(self, path: Path) -> bool:
        """
        Determine if a path should be excluded based on loaded patterns
        
        Args:
            path (Path): Path to check for exclusion
        
        Returns:
            bool: True if path should be excluded, False otherwise
        """
        # Similar to the existing implementation in core.py _is_excluded method
        pass

    def _is_targeted(self, path: Path) -> bool:
        """
        Check if a path is explicitly targeted for inclusion
        
        Args:
            path (Path): Path to check
        
        Returns:
            bool: True if path is targeted, False otherwise
        """
        for base_dir, spec in self.include_specs:
            if path.is_relative_to(base_dir):
                rel_path = str(path.relative_to(base_dir)).replace('\\', '/')
                if spec.match_file(rel_path):
                    return True
        return False

    def _is_peeked(self, path: Path) -> bool:
        """
        Check if a path is marked for peeking
        
        Args:
            path (Path): Path to check
        
        Returns:
            bool: True if path should be peeked, False otherwise
        """
        # Similar to the existing implementation in core.py _is_peek method
        pass

    def build_tree(self) -> ProjectTreeNode:
        """
        Build the entire project tree
        
        Returns:
            ProjectTreeNode: Root node of the processed project tree
        """
        root_node = ProjectTreeNode(
            path=self.root_dir, 
            is_dir=True,
            is_excluded=self._is_excluded(self.root_dir),
            is_targeted=self._is_targeted(self.root_dir)
        )

        def process_directory(current_dir: Path, parent_node: ProjectTreeNode, depth: int = 0):
            """Recursively process directory contents"""
            if self.max_depth and depth > self.max_depth:
                return

            try:
                entries = sorted(current_dir.iterdir(), key=lambda e: (not e.is_dir(), e.name.lower()))
                
                for entry in entries:
                    # Check exclusion and targeting
                    is_excluded = self._is_excluded(entry)
                    is_targeted = self._is_targeted(entry)
                    is_peeked = self._is_peeked(entry)

                    # Create node for this entry
                    node = ProjectTreeNode(
                        path=entry, 
                        is_dir=entry.is_dir(),
                        is_excluded=is_excluded,
                        is_targeted=is_targeted,
                        is_peeked=is_peeked
                    )

                    # Add content for files if applicable
                    if entry.is_file() and not is_excluded and not is_peeked:
                        try:
                            node.content = entry.read_text(encoding='utf-8')
                        except Exception as e:
                            if self.debug:
                                self.debug_log.append(f"Error reading {entry}: {e}")
                            node.content = None

                    # Recursively process directories
                    if entry.is_dir():
                        process_directory(entry, node, depth + 1)

                    # Add to parent's children if not excluded or is targeted
                    if not is_excluded or is_targeted:
                        parent_node.children.append(node)

            except PermissionError:
                if self.debug:
                    self.debug_log.append(f"Permission denied: {current_dir}")
            except Exception as e:
                if self.debug:
                    self.debug_log.append(f"Error processing directory {current_dir}: {e}")

        # Start processing from root
        process_directory(self.root_dir, root_node)

        return root_node