# tree_renderer.py
from pathlib import Path
from typing import List, Optional
from .project_tree import ProjectTreeNode

class MarkdownRenderer:
    def __init__(self, root_node: ProjectTreeNode, output_file: Optional[Path] = None, debug: bool = False):
        """
        Initialize the markdown renderer
        
        Args:
            root_node (ProjectTreeNode): Root of the project tree
            output_file (Optional[Path]): Optional output file path
            debug (bool): Enable debug logging
        """
        self.root_node = root_node
        self.output_file = output_file
        self.debug = debug
        self.debug_log: List[str] = []
    
    def _render_directory_structure(self, node: ProjectTreeNode, prefix: str = '', is_last: bool = True) -> List[str]:
        """
        Render directory structure as a tree
        
        Args:
            node (ProjectTreeNode): Current node to render
            prefix (str): Current indentation prefix
            is_last (bool): Whether this is the last entry in its parent
        
        Returns:
            List[str]: Lines representing the directory structure
        """
        lines = []
        
        # Determine connector and new prefix
        connector = '└── ' if is_last else '├── '
        new_prefix = prefix + ('    ' if is_last else '│   ')
        
        # Only render non-excluded nodes or targeted nodes
        if not node.is_excluded or node.is_targeted:
            # Add current node to tree
            lines.append(f"{prefix}{connector}{node.path.name}")
            
            # Recursively render children
            if node.is_dir:
                for i, child in enumerate(node.children):
                    child_is_last = (i == len(node.children) - 1)
                    lines.extend(
                        self._render_directory_structure(child, new_prefix, child_is_last)
                    )
        
        return lines
    
    def _render_file_contents(self, node: ProjectTreeNode) -> List[str]:
        """
        Render file contents section
        
        Args:
            node (ProjectTreeNode): Node to render
        
        Returns:
            List[str]: Lines representing file contents
        """
        contents = []
        separator = '-' * 80
        
        def process_node(current_node: ProjectTreeNode):
            # Render files that are not excluded
            if current_node.is_dir:
                for child in current_node.children:
                    process_node(child)
            elif not current_node.is_excluded or current_node.is_targeted:
                # Determine relative path from root
                try:
                    rel_path = current_node.path.relative_to(self.root_node.path)
                except ValueError:
                    rel_path = current_node.path
                
                # Render file header
                contents.append(f"\nFile: {rel_path}\n")
                contents.append(f"{separator}\n")
                
                # Render content
                if current_node.is_peeked:
                    contents.append("[Contents hidden]\n")
                elif current_node.content is None:
                    contents.append("[Error reading file]\n")
                else:
                    contents.append(f"{current_node.content}\n")
                
                contents.append(f"{separator}\n")
        
        process_node(node)
        return contents
    
    def render(self) -> str:
        """
        Render the entire project documentation
        
        Returns:
            str: Markdown-formatted project documentation
        """
        # Create output sections
        output = ["# Project Documentation\n"]
        
        # Debug information (if enabled)
        if self.debug and hasattr(self.root_node, 'debug_log'):
            output.append("## Debug Information\n```")
            output.extend(self.root_node.debug_log)
            output.append("```\n")
        
        # Directory structure
        output.append("\n## Directory Structure\n```\n")
        output.extend(self._render_directory_structure(self.root_node))
        output.append("\n```\n")
        
        # File contents
        output.append("\n## File Contents\n")
        output.extend(self._render_file_contents(self.root_node))
        
        # Optionally write to file
        if self.output_file:
            with open(self.output_file, 'w', encoding='utf-8') as f:
                f.write('\n'.join(output))
        
        return '\n'.join(output)